import React, {useRef, useState} from 'react';
import PropTypes from 'prop-types';
import { LinearProgress, Paper } from '@material-ui/core';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import { ContextMenu } from 'primereact/contextmenu';
import 'primeicons/primeicons.css';
import 'primereact/resources/themes/mdc-light-indigo/theme.css';
import 'primereact/resources/primereact.css';
import 'primeflex/primeflex.css';
import './CustomTable.css';

export function CustomTable(props) {
  const {
    columns, 
    rows, 
    handleRowClick, 
    itemLink, 
    totalCount, 
    lazyParams, 
    setLazyParams, 
    columnSort, 
    setColumnSort, 
    rowsPerPage, 
    loading=false, 
    paginator=true,
    paginatorTemplate
  } = props;
  
  const dynamicColumns = columns.map((col,i) => {
        return <Column key={col.id} field={col.id} header={col.label} style={{width: col.width, textAlign: col.textAlign}} body={col.body} sortable={col.sortable}/>;
    });

  const handleClick = (event) => {
    if(handleRowClick) {
      handleRowClick(event.data.id, event.data);
    }
  };

  const handleRightClickOpenInNewTab = (item) => {
    if(itemLink) {
        window.open(itemLink(item.id, item), "_blank");
    }
  };

  const onPage = (event) => {
    setLazyParams(event);
  }

  const onSort = (event) => {
    setColumnSort(event);
  }

  const rowClassName = (rowData) => {

    var resultObject = {};

    //if row is clickable -> change cursor by adding 'a new' className
    if (handleRowClick) {
      resultObject.isLink = true;
    }

    if (rowData.iocNotDeployed) {
      resultObject.iocNotDeployed = true;
    }

    if(rowData.csentryIdInvalid) {
      resultObject.csentryIdInvalid = true;
    }

    //for commands that are not run for current deployment should be grayed out
    if (rowData.notCurrentDeployment) {
      resultObject.notCurrentDeployment = true;
    }

    if (rowData.inconsistentState) {
      resultObject.inconsistentState = true;
      return resultObject;
    } else if (rowData.discrepancy) {
      resultObject.discrepancy = true;
      return resultObject;
    } else if (rowData.exporterUnknown) {
      resultObject.exporterUnknown = true;
      return resultObject;
    } else if (rowData.deploymentFailed) {
      resultObject.deploymentFailed = true;
    }

    return resultObject;
  }

  const cm = useRef(null);
  const menuModel = [
      {label: 'Open in new tab', icon: 'pi pi-fw pi-window-maximize', command: () => handleRightClickOpenInNewTab(selectedItem)}
  ];
  const [selectedItem, setSelectedItem] = useState(null);

  return (
    <div className="ccce-custom-table">
      {itemLink && <ContextMenu model={menuModel} ref={cm} onHide={() => setSelectedItem(null)}/>}
      <Paper>
        <DataTable
          className="p-ccce-custom-table"
          lazy
          first={lazyParams.page * lazyParams.rows}
          onPage={onPage}
          onSort={onSort}
          sortField={columnSort?.sortField} 
          sortOrder={columnSort?.sortOrder}
          value={rows}
          stripedRows
          scrollable 
          resizableColumns 
          columnResizeMode="fit"
          paginator={paginator}
          paginatorTemplate={paginatorTemplate ?? "CurrentPageReport FirstPageLink PrevPageLink PageLinks NextPageLink LastPageLink RowsPerPageDropdown"}
          currentPageReportTemplate="Showing {first} to {last} of {totalRecords}" 
          rows={lazyParams.rows}
          rowsPerPageOptions={rowsPerPage}
          rowHover={handleRowClick}
          onRowClick={(event) => handleClick(event)}
          rowClassName={rowClassName}
          totalRecords={totalCount}
          emptyMessage={loading ? <LinearProgress /> : "No records found"}
          contextMenuSelection={selectedItem}
          onContextMenuSelectionChange={e => setSelectedItem(e.value)}
          onContextMenu={itemLink ? (e => cm.current?.show(e.originalEvent)) : null}
        >
          {dynamicColumns}
        </DataTable>
      </Paper>
    </div>
  );
}

CustomTable.propTypes = {
  columns: PropTypes.arrayOf(PropTypes.shape({id: PropTypes.string.isRequired})).isRequired,
  rows: PropTypes.array.isRequired,
  handleRowClick: PropTypes.func,
};